"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const color_1 = require("@heroku-cli/color");
const core_1 = require("@oclif/core");
const core_2 = require("@salesforce/core");
const cli_ux_1 = require("cli-ux");
const date_fns_1 = require("date-fns");
const base_1 = require("../../../lib/base");
const flags_1 = require("../../../lib/flags");
const poll_for_result_1 = require("../../../lib/poll-for-result");
const utils_1 = require("../../../lib/utils");
core_2.Messages.importMessagesDirectory(__dirname);
const messages = core_2.Messages.loadMessages('@salesforce/plugin-functions', 'env.create.compute');
class EnvCreateCompute extends base_1.default {
    async run() {
        var _a, _b, _c;
        const { flags } = await this.parse(EnvCreateCompute);
        const alias = (_a = flags.alias) !== null && _a !== void 0 ? _a : flags.setalias;
        if (flags.setalias) {
            this.warn(messages.getMessage('flags.setalias.deprecation'));
        }
        // if `--connected-org` is null here, fetchOrg will pull the default org from the surrounding environment
        const org = await utils_1.fetchOrg(flags['connected-org']);
        const orgId = org.getOrgId();
        if (!(await this.isFunctionsEnabled(org))) {
            this.error(`The org you are attempting to create a compute environment for does not have the ${color_1.default.green('Functions')} feature enabled.\n` +
                '\n' +
                'Before you can create a compute environment, please:\n' +
                '1. Enable Functions in your DevHub org\n' +
                `2. Add ${color_1.default.green('Functions')} to the "features" list in your scratch org definition JSON file, e.g. "features": ["Functions"]`);
        }
        cli_ux_1.cli.action.start(`Creating compute environment for org ID ${orgId}`);
        const project = await utils_1.fetchSfdxProject();
        const projectName = project.name;
        if (!projectName) {
            this.error('No project name found in sfdx-project.json.');
        }
        const connection = org.getConnection();
        await poll_for_result_1.default(async () => {
            // This query allows us to verify that the org connection has actually been created before
            // attempting to create a compute environment. If we don't wait for this to happen, environment
            // creation will fail since Heroku doesn't yet know about the org
            let response;
            try {
                response = await connection.query(`SELECT
          Id,
          Status,
          Error
          FROM SfFunctionsConnection`);
            }
            catch (error) {
                // This is obviously heinous, but should only exist short-term until the move from `SfFunctionsConnection`
                // to `FunctionConnection` is complete. Once that's done, we can remove this and go back to a simple
                // query against `FunctionConnection`
                if (!error.message.includes("sObject type 'SfFunctionsConnection' is not supported.")) {
                    this.error(error);
                }
                response = await connection.query(`SELECT
            Id,
            Status,
            Error
            FROM FunctionConnection`);
            }
            // If it's a newly created org, we likely won't get anything back for the first few iterations,
            // we keep polling
            if (!response.records.length) {
                return false;
            }
            const record = response.records[0];
            // This error is also expected when working with a newly created org. This error just means
            // that the devhub hasn't yet enabled functions on the new org (this is an automated async process
            // so it takes a bit of time)
            if (record.Error === 'Enable Salesforce Functions from Setup Page') {
                return false;
            }
            // If there is any other error besides the one mentioned above, something is actually wrong
            // and we should bail
            if (record.Error) {
                this.error(record.Error);
            }
            // This is the go signal. Once we have this status it means that the connection is fully up
            // and running, and we are good to create a compute environment.
            return record.Status === 'TrustedBiDirection';
        });
        try {
            const { data: app } = await this.client.post(`/sales-org-connections/${orgId}/apps`, {
                headers: {
                    Accept: 'application/vnd.heroku+json; version=3.evergreen',
                },
                data: {
                    sfdx_project_name: projectName,
                },
            });
            cli_ux_1.cli.action.stop();
            this.log(`New compute environment created with ID ${app.name}`);
            cli_ux_1.cli.action.start('Connecting environments');
            if (alias) {
                this.info.aliases.set(alias, app.id);
                await this.info.write();
            }
            cli_ux_1.cli.action.stop();
            this.log(alias
                ? `Your compute environment with local alias ${color_1.default.cyan(alias)} is ready.`
                : 'Your compute environment is ready.');
        }
        catch (error) {
            const DUPLICATE_PROJECT_MESSAGE = 'This org is already connected to a compute environment for this project';
            // If environment creation fails because an environment already exists for this org and project
            // we want to fetch the existing environment so that we can point the user to it
            if ((_c = (_b = error.body) === null || _b === void 0 ? void 0 : _b.message) === null || _c === void 0 ? void 0 : _c.includes(DUPLICATE_PROJECT_MESSAGE)) {
                cli_ux_1.cli.action.stop('error!');
                const app = await utils_1.fetchAppForProject(this.client, projectName, org.getUsername());
                this.log(`${DUPLICATE_PROJECT_MESSAGE}:`);
                this.log(`Compute Environment ID: ${app.name}`);
                if (app.created_at) {
                    this.log(`Created on: ${date_fns_1.format(new Date(app.created_at), 'E LLL d HH:mm:ss O y')}`);
                }
                return;
            }
            throw error;
        }
    }
}
exports.default = EnvCreateCompute;
EnvCreateCompute.description = messages.getMessage('summary');
EnvCreateCompute.examples = messages.getMessages('examples');
EnvCreateCompute.flags = {
    'connected-org': flags_1.FunctionsFlagBuilder.connectedOrg(),
    alias: core_1.Flags.string({
        char: 'a',
        description: messages.getMessage('flags.alias.summary'),
        exclusive: ['setalias'],
    }),
    setalias: core_1.Flags.string({
        char: 'a',
        description: messages.getMessage('flags.alias.summary'),
        exclusive: ['alias'],
        hidden: true,
    }),
};
